"use strict";
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);

// src/index.ts
var src_exports = {};
__export(src_exports, {
  plugin: () => plugin
});
module.exports = __toCommonJS(src_exports);

// src/code-generator.ts
function generatePreludeCode(config, typeInfos) {
  const joinedTypeNames = typeInfos.filter(({ type }) => type === "object").map(({ name }) => `  ${name},
`).join("");
  const code = `
import {
  type DefineTypeFactoryInterface${config.nonOptionalDefaultFields ? "Required" : ""},
  defineTypeFactory,
} from '@mizdra/graphql-codegen-typescript-fabbrica/helper';
import type {
  Maybe,
${joinedTypeNames}} from '${config.typesFile}';

export * from '@mizdra/graphql-codegen-typescript-fabbrica/helper';
  `.trim();
  return `${code}
`;
}
function generateOptionalTypeDefinitionCode(typeInfo) {
  if (typeInfo.type === "object") {
    const { name, fields } = typeInfo;
    const comment = typeInfo.comment ?? "";
    const joinedPropDefinitions = fields.map((field) => {
      const comment2 = field.comment ? `  ${field.comment}` : "";
      return `${comment2}  ${field.name}?: ${field.typeString};`;
    }).join("\n");
    return `
${comment}export type Optional${name} = {
${joinedPropDefinitions}
};
`.trimStart();
  } else {
    const { name, possibleTypes } = typeInfo;
    const comment = typeInfo.comment ?? "";
    const joinedPossibleTypes = possibleTypes.map((type) => `Optional${type}`).join(" | ");
    return `
${comment}export type Optional${name} = ${joinedPossibleTypes};
`.trimStart();
  }
}
function generateTypeFactoryCode({ nonOptionalDefaultFields }, typeInfo) {
  const { name } = typeInfo;
  return `
/**
 * Define factory for {@link ${name}} model.
 *
 * @param options
 * @returns factory {@link ${name}FactoryInterface}
 */
export const define${name}Factory: DefineTypeFactoryInterface${nonOptionalDefaultFields ? "Required" : ""}<
  Optional${name},
  {}
> = defineTypeFactory;
`.trimStart();
}
function generateCode(config, typeInfos) {
  let code = "";
  code += generatePreludeCode(config, typeInfos);
  code += "\n";
  for (const typeInfo of typeInfos) {
    code += generateOptionalTypeDefinitionCode(typeInfo);
    code += "\n";
    if (typeInfo.type === "object") {
      code += generateTypeFactoryCode(config, typeInfo);
      code += "\n";
    }
  }
  return code;
}

// src/config.ts
var import_visitor_plugin_common = require("@graphql-codegen/visitor-plugin-common");
function validateConfig(rawConfig) {
  if (typeof rawConfig !== "object" || rawConfig === null) {
    throw new Error("`options` must be an object");
  }
  if (!("typesFile" in rawConfig)) {
    throw new Error("`option.typesFile` is required");
  }
  if (typeof rawConfig["typesFile"] !== "string") {
    throw new Error("`options.typesFile` must be a string");
  }
  if ("skipTypename" in rawConfig && typeof rawConfig["skipTypename"] !== "boolean") {
    throw new Error("`options.skipTypename` must be a boolean");
  }
  if ("skipIsAbstractType" in rawConfig && typeof rawConfig["skipIsAbstractType"] !== "boolean") {
    throw new Error("`options.skipIsAbstractType` must be a boolean");
  }
  if ("nonOptionalDefaultFields" in rawConfig && typeof rawConfig["nonOptionalDefaultFields"] !== "boolean") {
    throw new Error("`options.nonOptionalDefaultFields` must be a boolean");
  }
  if ("typesPrefix" in rawConfig && typeof rawConfig["typesPrefix"] !== "string") {
    throw new Error("`options.typesPrefix` must be a string");
  }
  if ("typesSuffix" in rawConfig && typeof rawConfig["typesSuffix"] !== "string") {
    throw new Error("`options.typesSuffix` must be a string");
  }
}
function normalizeConfig(rawConfig) {
  return {
    typesFile: rawConfig.typesFile,
    skipTypename: rawConfig.skipTypename ?? false,
    skipIsAbstractType: rawConfig.skipIsAbstractType ?? true,
    nonOptionalDefaultFields: rawConfig.nonOptionalDefaultFields ?? false,
    typesPrefix: rawConfig.typesPrefix ?? "",
    typesSuffix: rawConfig.typesSuffix ?? "",
    convert: rawConfig.namingConvention ? (0, import_visitor_plugin_common.convertFactory)({ namingConvention: rawConfig.namingConvention }) : (0, import_visitor_plugin_common.convertFactory)({})
  };
}

// src/schema-scanner.ts
var import_visitor_plugin_common2 = require("@graphql-codegen/visitor-plugin-common");
var import_graphql = require("graphql/index.js");
function clearOptional(str) {
  if (str.startsWith("Maybe")) {
    return str.replace(/Maybe<(.*?)>$/u, "$1");
  }
  return str;
}
function convertName(node, config) {
  let convertedName = "";
  convertedName += config.typesPrefix;
  convertedName += config.convert(node);
  convertedName += config.typesSuffix;
  return convertedName;
}
function isTypeBasedOnUserDefinedType(type, userDefinedTypeNames) {
  if ((0, import_graphql.isNonNullType)(type)) {
    return isTypeBasedOnUserDefinedType(type.ofType, userDefinedTypeNames);
  } else if ((0, import_graphql.isListType)(type)) {
    return isTypeBasedOnUserDefinedType(type.ofType, userDefinedTypeNames);
  } else {
    return userDefinedTypeNames.includes(type.name);
  }
}
function parseTypeNode(type, config) {
  if ((0, import_graphql.isNonNullType)(type)) {
    return clearOptional(parseTypeNode(type.ofType, config));
  } else if ((0, import_graphql.isListType)(type)) {
    return `Maybe<${parseTypeNode(type.ofType, config)}[]>`;
  } else {
    return `Maybe<Optional${convertName(type.name, config)}>`;
  }
}
function parseFieldOrInputField(field, convertedTypeName, config, userDefinedTypeNames) {
  const comment = field.description ? (0, import_visitor_plugin_common2.transformComment)(field.description) : void 0;
  if (isTypeBasedOnUserDefinedType(field.type, userDefinedTypeNames)) {
    return { typeString: `${parseTypeNode(field.type, config)} | undefined`, comment };
  } else {
    return { typeString: `${convertedTypeName}['${field.name}'] | undefined`, comment };
  }
}
function parseObjectTypeOrInputObjectType(type, config, userDefinedTypeNames, getAbstractTypeNames) {
  const originalTypeName = type.name;
  const convertedTypeName = convertName(originalTypeName, config);
  const comment = type.description ? (0, import_visitor_plugin_common2.transformComment)(type.description) : void 0;
  const abstractTypeNames = (0, import_graphql.isObjectType)(type) ? getAbstractTypeNames(type) : [];
  const fields = Object.values(type.getFields());
  return {
    type: "object",
    name: convertedTypeName,
    fields: [
      ...!config.skipTypename ? [{ name: "__typename", typeString: `'${originalTypeName}'` }] : [],
      ...!config.skipIsAbstractType ? abstractTypeNames.map((name) => ({ name: `__is${name}`, typeString: `'${originalTypeName}'` })) : [],
      ...fields.map((field) => ({
        name: field.name,
        ...parseFieldOrInputField(field, convertedTypeName, config, userDefinedTypeNames)
      }))
    ],
    comment
  };
}
function getTypeInfos(config, schema) {
  const types = Object.values(schema.getTypeMap());
  const userDefinedTypes = types.filter((type) => !(0, import_graphql.isIntrospectionType)(type) && !(0, import_graphql.isSpecifiedScalarType)(type)).filter((type) => (0, import_graphql.isObjectType)(type) || (0, import_graphql.isInputObjectType)(type) || (0, import_graphql.isInterfaceType)(type) || (0, import_graphql.isUnionType)(type));
  const objectTypes = userDefinedTypes.filter(import_graphql.isObjectType);
  const unionTypes = userDefinedTypes.filter(import_graphql.isUnionType);
  function getAbstractTypeNames(type) {
    const interfaceNames = type.getInterfaces().map((i) => i.name);
    const unionNames = unionTypes.filter((union) => union.getTypes().some((member) => member.name === type.name)).map((union) => union.name);
    return [...interfaceNames, ...unionNames];
  }
  const userDefinedTypeNames = userDefinedTypes.map((type) => type.name);
  return userDefinedTypes.map((type) => {
    if ((0, import_graphql.isObjectType)(type) || (0, import_graphql.isInputObjectType)(type)) {
      return parseObjectTypeOrInputObjectType(type, config, userDefinedTypeNames, getAbstractTypeNames);
    } else if ((0, import_graphql.isInterfaceType)(type)) {
      return {
        type: "abstract",
        name: convertName(type.name, config),
        possibleTypes: objectTypes.filter((objectType) => objectType.getInterfaces().some((i) => i.name === type.name)).map((objectType) => convertName(objectType.name, config)),
        comment: type.description ? (0, import_visitor_plugin_common2.transformComment)(type.description) : void 0
      };
    } else {
      return {
        type: "abstract",
        name: convertName(type.name, config),
        possibleTypes: type.getTypes().map((member) => convertName(member.name, config)),
        comment: type.description ? (0, import_visitor_plugin_common2.transformComment)(type.description) : void 0
      };
    }
  });
}

// src/index.ts
var plugin = (schema, _documents, config, _info) => {
  validateConfig(config);
  const normalizedConfig = normalizeConfig(config);
  const typeInfos = getTypeInfos(normalizedConfig, schema);
  const code = generateCode(normalizedConfig, typeInfos);
  return code;
};
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  plugin
});
//# sourceMappingURL=index.cjs.map